@@ -2,8 +2,6 @@
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 '''
 
-import tempfile
-
 import pytest
 import numpy as np
 
@@ -17,17 +15,17 @@ from ...tools.misc import skipUnlessModule
         
 @skipUnlessModule("h5py")
 @pytest.mark.parametrize('compression', [True, False])
-def test_storage_persistence(compression):
+def test_storage_persistence(compression, tmp_path):
     """ test writing to persistent trackers """
     dim = 5
     grid = UnitGrid([dim])
     field = ScalarField(grid)
 
-    file = tempfile.NamedTemporaryFile(suffix='.hdf5')
+    path = tmp_path / f'test_storage_persistence_{compression}.hdf5'
     
     # write some data 
     for write_mode in ['append', 'truncate_once', 'truncate']:
-        storage = FileStorage(file.name, info={'a': 1},
+        storage = FileStorage(path, info={'a': 1},
                               write_mode=write_mode,
                               compression=compression)
         
@@ -50,7 +48,7 @@ def test_storage_persistence(compression):
         storage.close()
         
         # read the data
-        storage = FileStorage(file.name)
+        storage = FileStorage(path)
         if write_mode == 'truncate':
             np.testing.assert_array_equal(storage.times, np.array([2]))
             np.testing.assert_array_equal(np.ravel(storage.data),
@@ -70,11 +68,11 @@ def test_storage_persistence(compression):
      
 @skipUnlessModule("h5py")
 @pytest.mark.parametrize('compression', [True, False])
-def test_simulation_persistence(compression):
+def test_simulation_persistence(compression, tmp_path):
     """ test whether a tracker can accurately store information about
     simulation """
-    file = tempfile.NamedTemporaryFile(suffix='.hdf5')
-    storage = FileStorage(file.name, compression=compression)
+    path = tmp_path / 'test_simulation_persistence.hdf5'
+    storage = FileStorage(path, compression=compression)
      
     # write some simulation data
     pde = DiffusionPDE()
@@ -85,7 +83,7 @@ def test_simulation_persistence(compression):
     storage.close()        
      
     # read the data
-    storage = FileStorage(file.name)
+    storage = FileStorage(path)
     np.testing.assert_almost_equal(storage.times, [0, 0.05, 0.1])
     data = np.array(storage.data)
     assert data.shape == (3,) + state.data.shape
@@ -98,13 +96,13 @@ def test_simulation_persistence(compression):
 
 @skipUnlessModule("h5py")
 @pytest.mark.parametrize('compression', [True, False])
-def test_storage_fixed_size(compression):
+def test_storage_fixed_size(compression, tmp_path):
     """ test setting fixed size of FileStorage objects """
     c = ScalarField(UnitGrid([2]), data=1)
 
     for fixed in [True, False]:
-        file = tempfile.NamedTemporaryFile(suffix='.hdf5')
-        storage = FileStorage(file.name, max_length=1 if fixed else None,
+        path = tmp_path / f'test_storage_fixed_size_{fixed}.hdf5'
+        storage = FileStorage(path, max_length=1 if fixed else None,
                               compression=compression)
         assert len(storage) == 0
         
@@ -126,11 +124,12 @@ def test_storage_fixed_size(compression):
             
             
 @skipUnlessModule("h5py")
-def test_appending():
+def test_appending(tmp_path):
     """ test the appending data """
+    path = tmp_path / 'test_appending.hdf5'
+    
     c = ScalarField(UnitGrid([2]), data=1)
-    file = tempfile.NamedTemporaryFile(suffix='.hdf5')
-    storage = FileStorage(file.name)
+    storage = FileStorage(path)
     storage.start_writing(c)
     assert len(storage) == 0
     storage.append(c.data, 0)
@@ -138,7 +137,7 @@ def test_appending():
     assert len(storage) == 1
     storage.close()
                 
-    storage2 = FileStorage(file.name, write_mode='append')
+    storage2 = FileStorage(path, write_mode='append')
     storage2.start_writing(c)
     storage2.append(c.data, 1)
     storage2.close()
@@ -148,11 +147,12 @@ def test_appending():
 
             
 @skipUnlessModule("h5py")
-def test_keep_opened():
+def test_keep_opened(tmp_path):
     """ test the keep opened option """
+    path = tmp_path / 'test_keep_opened.hdf5'
+    
     c = ScalarField(UnitGrid([2]), data=1)
-    file = tempfile.NamedTemporaryFile(suffix='.hdf5')
-    storage = FileStorage(file.name, keep_opened=False)
+    storage = FileStorage(path, keep_opened=False)
     storage.start_writing(c)
     assert len(storage) == 0
     storage.append(c.data, 0)
@@ -162,7 +162,7 @@ def test_keep_opened():
     storage.append(c.data, 1)
     assert len(storage) == 2
                 
-    storage2 = FileStorage(file.name, write_mode='append')
+    storage2 = FileStorage(path, write_mode='append')
     assert storage.times == storage2.times
     assert storage.data == storage2.data
     storage.close()  # close the old storage to enable writing here
